package net.sf.corn.gate.jsonrpc;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.HttpCookie;
import java.net.URI;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;

import net.sf.corn.converter.ConversionException;
import net.sf.corn.converter.ParsingException;
import net.sf.corn.converter.json.JsTypeComplex;
import net.sf.corn.converter.json.JsTypeList;
import net.sf.corn.converter.json.JsTypeObject;
import net.sf.corn.converter.json.JsonConverter;
import net.sf.corn.converter.json.JsonStringParser;
import net.sf.corn.httpclient.HttpClient;
import net.sf.corn.httpclient.HttpClient.HTTP_METHOD;
import net.sf.corn.httpclient.HttpResponse;
import net.sf.corn.httpclient.TSAuthenticator;

import org.springframework.beans.ConversionNotSupportedException;

/**
 * This class is java client implementation to access remote JSON-RPC services. 
 * Class supports authentication with BASIC,DIGEST & NTLM methods. 
 * <br>Usage: 
 * <code><br>
 * JsonRpcClient client  = new JsonRpcClient(new URI("http://localhost:8080/project/json"));<br>
 * JsonRpcResponse resp = client.callAService("echoBean","echo", null, "test");
 * </code>
 * @author Serhat Dirik
 * @see TSAuthenticator
 *
 */
public class JsonRpcClient {

	public final static String FIELD_JSONRPC_PARAMS = "params";
	public final static String FIELD_JSONRPC_HEADER = "header";
	public final static String FIELD_JSONRPC_ID     = "id";
	private HttpClient  client = null;
	
	/**
	 * Client constructor for http connections to call json rpc
	 * @param servletURI JSON Request Response Servlet Url
	 */
	public JsonRpcClient(URI servletURI){
		 client = new HttpClient(servletURI);
	}
  
	
	
	public String getCharset() {
		return client.getCharset();
	}



	public void setCharset(String charset) throws UnsupportedEncodingException {
		client.setCharset(charset);
	}



	public void putAdditionalRequestProperty(String name, String value) {
		client.putAdditionalRequestProperty(name, value);
	}



	public void clearAdditionalRequestProperties() {
		client.clearAdditionalRequestProperties();
	}



	public void setReadTimeout(int miliseconds) {
		client.setReadTimeout(miliseconds);
	}



	public int getReadTimeout() {
		return client.getReadTimeout();
	}



	public void setProxyCredentials(String uname, String password) {
		client.setProxyCredentials(uname, password);
	}



	public void setCredentials(String uname, String password) {
		client.setCredentials(uname, password);
	}



	public void clearCookies() {
		client.clearCookies();
	}



	public List<HttpCookie> getCookies() {
		return client.getCookies();
	}



	public String getCookiesString() {
		return client.getCookiesString();
	}



	public HttpCookie getCookie(String name) {
		return client.getCookie(name);
	}



	public void addCookies(String cookies) {
		client.addCookies(cookies);
	}



	public void addCookie(HttpCookie cookie) {
		client.addCookie(cookie);
	}



	public void removeCookie(String name) {
		client.removeCookie(name);
	}



	public void setCookies(String cookies) {
		client.setCookies(cookies);
	}



	public boolean isFollowRedirects() {
		return client.isFollowRedirects();
	}



	public void setFollowRedirects(boolean followRedirects) {
		client.setFollowRedirects(followRedirects);
	}



	/**
	 * Calls the remote the servlet for the given service and the operation.This method generates and uses a UUID as 
	 *  message identifier.
	 * @param service    service name
	 * @param operation  operation name 
	 * @return the result will be wrapped in the JSONResponse as error or response
	 * @throws IOException throws when an exception happens when posting request to the given url
	 * @throws ParsingException 
	 * @throws ConversionException 
	 * @throws ConversionNotSupportedException 
	 * @throws JSONException throws if an exception happens when preparing request
	 * @throws ExConversion throws if an exception happens during JSON-Java transformations or vice versa
	 */
	public JsonRpcResponse callAService(String service,String operation) 
		throws IOException, ConversionNotSupportedException, ConversionException, ParsingException{
 
			String requestId = UUID.randomUUID().toString();
			return callAServiceWithID(requestId, service, operation, null, new Object[]{});
			
	}
	
	/**
	 * Calls the remote the servlet for the given service and the operation.This method generates and uses a UUID as 
	 *  message identifier.
	 * @param service    service name
	 * @param operation  operation name 
	 * @param headers    Header part of the message as given string,object map
	 * @return the result will be wrapped in the JSONResponse as error or response
	 * @throws IOException throws when an exception happens when posting request to the given url
	 * @throws ParsingException 
	 * @throws ConversionException 
	 * @throws ConversionNotSupportedException 
	 * @throws JSONException throws if an exception happens when preparing request
	 * @throws ExConversion throws if an exception happens during JSON-Java transformations or vice versa
	 */
	public JsonRpcResponse callAService(String service,String operation,Map<String,Object> headers) 
		throws IOException, ConversionNotSupportedException, ConversionException, ParsingException{
 
			String requestId = UUID.randomUUID().toString();
			return callAServiceWithID(requestId, service, operation, headers, new Object[]{});
			
	}
	
	
	/**
	 * Calls the remote the servlet for the given service and the operation.This method generates and uses a UUID as 
	 *  message identifier.
	 * @param service    service name
	 * @param operation  operation name 
	 * @param headers    Header part of the message as given string,object map
	 * @param parameters operation parameters, message payload 
	 * @return the result will be wrapped in the JSONResponse as error or response
	 * @throws IOException throws when an exception happens when posting request to the given url
	 * @throws ParsingException 
	 * @throws ConversionException 
	 * @throws ConversionNotSupportedException 
	 * @throws JSONException throws if an exception happens when preparing request
	 * @throws ExConversion throws if an exception happens during JSON-Java transformations or vice versa
	 */
	public JsonRpcResponse callAService(String service,String operation,Map<String,Object> headers,Object...parameters) 
		throws IOException, ConversionNotSupportedException, ConversionException, ParsingException{
 
			String requestId = UUID.randomUUID().toString();
			return callAServiceWithID(requestId, service, operation, headers, parameters);
			
	}
	
	
	/**
	 * Calls the remote the servlet for the given service and the operation.
	 * @param service    service name
	 * @param operation  operation name 
	 * @return the result will be wrapped in the JSONResponse as error or response
	 * @throws IOException throws when an exception happens when posting request to the given url
	 * @throws ParsingException 
	 * @throws ConversionException 
	 * @throws JSONException throws if an exception happens when preparing request
	 * @throws ExConversion 
	 * @throws TransformationException throws if an exception happens during JSON-Java transformations or vice versa
	 */
	public JsonRpcResponse callAServiceWithID(String id,String service,String operation) 
		throws IOException, ConversionNotSupportedException, ConversionException, ParsingException {
		return callAServiceWithID(id, service, operation, null, new Object[]{});
	}
	
	/**
	 * Calls the remote the servlet for the given service and the operation.
	 * @param service    service name
	 * @param operation  operation name 
	 * @param headers    Header part of the message as given string,object map
	 * @return the result will be wrapped in the JSONResponse as error or response
	 * @throws IOException throws when an exception happens when posting request to the given url
	 * @throws ParsingException 
	 * @throws ConversionException 
	 * @throws JSONException throws if an exception happens when preparing request
	 * @throws ExConversion 
	 * @throws TransformationException throws if an exception happens during JSON-Java transformations or vice versa
	 */
	public JsonRpcResponse callAServiceWithID(String id,String service,String operation,Map<String,Object> headers) 
		throws IOException, ConversionNotSupportedException, ConversionException, ParsingException {
		return callAServiceWithID(id, service, operation, headers, new Object[]{});
	}
	
	/**
	 * Calls the remote the servlet for the given service and the operation.
	 * @param service    service name
	 * @param operation  operation name 
	 * @param headers    Header part of the message as given string,object map
	 * @param parameters operation parameters, message payload 
	 * @return the result will be wrapped in the JSONResponse as error or response
	 * @throws IOException throws when an exception happens when posting request to the given url
	 * @throws ConversionException 
	 * @throws ParsingException 
	 * @throws JSONException throws if an exception happens when preparing request
	 * @throws ExConversion 
	 * @throws TransformationException throws if an exception happens during JSON-Java transformations or vice versa
	 */
	public JsonRpcResponse callAServiceWithID(String id,String service,String operation,Map<String,Object> headers,Object...parameters) 
		throws IOException, ConversionNotSupportedException, ConversionException, ParsingException {
			JsonRpcRequest request = new JsonRpcRequest();
			request.setServiceName(service);
			request.setOperationName(operation);
			request.setRequestId(id);
			
			JsonConverter transformer = new JsonConverter();
			if(parameters != null){
				JsTypeList jsparameters  = new JsTypeList();
				for (Object object : parameters) {
					jsparameters.add(transformer.toTargetType(object));
				}
				request.setParameters(jsparameters);
			}
			
			if(headers != null){
				Map<String,Object> jsMap = new HashMap<String, Object>(headers.size());
				for (String key: headers.keySet()) {
					jsMap.put(key, transformer.toTargetType(headers.get(key)));
				}
				request.setHeader(jsMap);
			}
			
			String sendData = request.toString();
			HttpResponse formResponse =  client.sendData(HTTP_METHOD.POST, sendData);
			JsonRpcResponse resp= null;
			if(formResponse.hasError()){
				resp = new JsonRpcResponse(id,"ConnectionException"+ String.valueOf(formResponse.getCode()),formResponse.getMessage());
			}else{
				JsTypeObject res = JsonStringParser.parseJsonString(formResponse.getData());
				resp= new JsonRpcResponse((JsTypeComplex) res);
			}
			
			
			return resp;
			
	}
	
}
